import { _InstanceType } from '@aws-sdk/client-ec2'
import { GPUSpec, VMSpec } from '../../Driver'

type GPUDetail = {
  count: number
  model: NonNullable<GPUSpec['model']>
}

export type InstanceDetail = {
  type: _InstanceType
  cpuCount: number
  ramGiB: number
  cpuArchitecture: NonNullable<VMSpec['cpu_architecture']>
  gpuDetail?: GPUDetail
}

export const instanceDetails: InstanceDetail[] = [
  { type: 't3.nano', cpuCount: 2, ramGiB: 0.5, cpuArchitecture: 'x64' },
  { type: 't3.micro', cpuCount: 2, ramGiB: 1, cpuArchitecture: 'x64' },
  { type: 't3.small', cpuCount: 2, ramGiB: 2, cpuArchitecture: 'x64' },
  { type: 't3.medium', cpuCount: 2, ramGiB: 4, cpuArchitecture: 'x64' },
  { type: 't3.large', cpuCount: 2, ramGiB: 8, cpuArchitecture: 'x64' },
  { type: 't3.xlarge', cpuCount: 4, ramGiB: 16, cpuArchitecture: 'x64' },
  { type: 't3.2xlarge', cpuCount: 8, ramGiB: 32, cpuArchitecture: 'x64' },

  { type: 't4g.nano', cpuCount: 2, ramGiB: 0.5, cpuArchitecture: 'arm64' },
  { type: 't4g.micro', cpuCount: 2, ramGiB: 1, cpuArchitecture: 'arm64' },
  { type: 't4g.small', cpuCount: 2, ramGiB: 2, cpuArchitecture: 'arm64' },
  { type: 't4g.medium', cpuCount: 2, ramGiB: 4, cpuArchitecture: 'arm64' },
  { type: 't4g.large', cpuCount: 2, ramGiB: 8, cpuArchitecture: 'arm64' },
  { type: 't4g.xlarge', cpuCount: 4, ramGiB: 16, cpuArchitecture: 'arm64' },
  { type: 't4g.2xlarge', cpuCount: 8, ramGiB: 32, cpuArchitecture: 'arm64' },

  { type: 'g5.xlarge', cpuCount: 4, ramGiB: 16, cpuArchitecture: 'x64', gpuDetail: { count: 1, model: 'a10' } },
  { type: 'g5.2xlarge', cpuCount: 8, ramGiB: 32, cpuArchitecture: 'x64', gpuDetail: { count: 1, model: 'a10' } },
  { type: 'g5.4xlarge', cpuCount: 16, ramGiB: 64, cpuArchitecture: 'x64', gpuDetail: { count: 1, model: 'a10' } },
  { type: 'g5.8xlarge', cpuCount: 32, ramGiB: 128, cpuArchitecture: 'x64', gpuDetail: { count: 1, model: 'a10' } },
  {
    type: 'g5.16xlarge',
    cpuCount: 64,
    ramGiB: 256,
    cpuArchitecture: 'x64',
    gpuDetail: { count: 1, model: 'a10' },
  },
  {
    type: 'g5.12xlarge',
    cpuCount: 48,
    ramGiB: 192,
    cpuArchitecture: 'x64',
    gpuDetail: { count: 4, model: 'a10' },
  },
  {
    type: 'g5.24xlarge',
    cpuCount: 96,
    ramGiB: 384,
    cpuArchitecture: 'x64',
    gpuDetail: { count: 4, model: 'a10' },
  },
  {
    type: 'g5.48xlarge',
    cpuCount: 192,
    ramGiB: 768,
    cpuArchitecture: 'x64',
    gpuDetail: { count: 8, model: 'a10' },
  },
  {
    type: 'p3.2xlarge',
    cpuCount: 8,
    ramGiB: 61,
    cpuArchitecture: 'x64',
    gpuDetail: { count: 1, model: 'v100' },
  },
  {
    type: 'p3.8xlarge',
    cpuCount: 32,
    ramGiB: 244,
    cpuArchitecture: 'x64',
    gpuDetail: { count: 4, model: 'v100' },
  },
  {
    type: 'p3.16xlarge',
    cpuCount: 64,
    ramGiB: 488,
    cpuArchitecture: 'x64',
    gpuDetail: { count: 8, model: 'v100' },
  },
  {
    type: 'p4d.24xlarge',
    cpuCount: 96,
    ramGiB: 1152,
    cpuArchitecture: 'x64',
    gpuDetail: { count: 8, model: 'a100' },
  },

  {
    type: 'p5.48xlarge',
    cpuCount: 192,
    ramGiB: 2048,
    cpuArchitecture: 'x64',
    gpuDetail: { count: 8, model: 'h100' },
  },
]

export type AmiDetail = {
  ids: Record<string, string>
  cpuArchitecture: NonNullable<VMSpec['cpu_architecture']>
  baseImageType: NonNullable<VMSpec['base_image_type']>
  // If unset, we assume that the AMI supports all instance types for the given CPU architecture.
  supportedInstanceTypes?: _InstanceType[]
  sshUsername: string
  rootVolumeDeviceName: string
  rootVolumeSize: number
}

export const amiDetails: AmiDetail[] = [
  {
    ids: {
      'af-south-1': 'ami-0c340a880c01f45dc',
      'ap-east-1': 'ami-03e011b20cc3eb9ad',
      'ap-northeast-1': 'ami-08c58abb4bd7fdf48',
      'ap-northeast-2': 'ami-0782173ac8dd43ead',
      'ap-northeast-3': 'ami-04088c641c86a8cd3',
      'ap-south-1': 'ami-0f7749c2155b4aa41',
      'ap-south-2': 'ami-03dace10096b085c0',
      'ap-southeast-1': 'ami-06e25c4a45918a7ad',
      'ap-southeast-2': 'ami-0e1dab4992930dfc5',
      'ap-southeast-3': 'ami-06a331418b1986707',
      'ap-southeast-4': 'ami-01920df6764a616d2',
      'ca-central-1': 'ami-064d502aeb1744e24',
      'ca-west-1': 'ami-0d7891a6c504ca3ba',
      'eu-central-1': 'ami-0e626c31414223120',
      'eu-central-2': 'ami-0388736a84da3da43',
      'eu-north-1': 'ami-0735de43bc09fe101',
      'eu-south-1': 'ami-0db6556eb549643d9',
      'eu-south-2': 'ami-05ba2fddf42cb711a',
      'eu-west-1': 'ami-04db4ad9b30430256',
      'eu-west-2': 'ami-04ff18fe839e02019',
      'eu-west-3': 'ami-07254a4b88b1dc88d',
      'il-central-1': 'ami-0285c3a2937c2f32b',
      'me-central-1': 'ami-07f9bd8efb6fef0c1',
      'me-south-1': 'ami-033a15bfac8dd3f7b',
      'sa-east-1': 'ami-08daa9eaf8b0ea93e',
      'us-east-1': 'ami-02269465a4b1f7ef5',
      'us-east-2': 'ami-006309fcfbf4b30d6',
      'us-west-1': 'ami-0d1e92cd5c4f492eb',
      'us-west-2': 'ami-0eabbc35930d487e3',
    },
    cpuArchitecture: 'x64',
    baseImageType: 'debian-12',
    sshUsername: 'admin',
    rootVolumeDeviceName: '/dev/xvda',
    rootVolumeSize: 8, // This is the default root volume size for this AMI.
  },
  {
    ids: {
      'af-south-1': 'ami-0786aa0aaa5eb84e6',
      'ap-east-1': 'ami-056b7fe63f6e13a0e',
      'ap-northeast-1': 'ami-094e3ec61bd7f784c',
      'ap-northeast-2': 'ami-083b4d80a80f5b23d',
      'ap-northeast-3': 'ami-0a133923699fcd599',
      'ap-south-1': 'ami-0871c05b470620de8',
      'ap-south-2': 'ami-0705e646d070674b8',
      'ap-southeast-1': 'ami-0373adef92f28feef',
      'ap-southeast-2': 'ami-02b2b8946d0491f71',
      'ap-southeast-3': 'ami-02d3b96bcd16d4965',
      'ap-southeast-4': 'ami-0d03d6069fa6ec01c',
      'ca-central-1': 'ami-0b59026f3f2752cf0',
      'ca-west-1': 'ami-06f3f816429f577dd',
      'eu-central-1': 'ami-0f89740d2b88635ad',
      'eu-central-2': 'ami-0f29c90015f255aaa',
      'eu-north-1': 'ami-01123efb21a1047f1',
      'eu-south-1': 'ami-0d9b8c7d31cfe891c',
      'eu-south-2': 'ami-09188178d336e24bf',
      'eu-west-1': 'ami-0d7bb726b0bf93197',
      'eu-west-2': 'ami-0933c6159641f735c',
      'eu-west-3': 'ami-0bf90bff37ed95f27',
      'il-central-1': 'ami-04dcffde7317ec0e0',
      'me-central-1': 'ami-037e0824b6b72f89f',
      'me-south-1': 'ami-0eddcbb84cf1285a9',
      'sa-east-1': 'ami-054436877ea027ba6',
      'us-east-1': 'ami-0e4d3a17b8dcf601f',
      'us-east-2': 'ami-07f23827ddd9e09e0',
      'us-west-1': 'ami-07b71ab501b45057e',
      'us-west-2': 'ami-093939e1f4317142c',
    },
    cpuArchitecture: 'arm64',
    baseImageType: 'debian-12',
    sshUsername: 'admin',
    rootVolumeDeviceName: '/dev/xvda',
    rootVolumeSize: 8, // This is the default root volume size for this AMI.
  },
  {
    ids: {
      'ap-south-1': 'ami-027f4b772df3e367c',
      'eu-north-1': 'ami-0feaf98c2c48d0ab8',
      'eu-west-3': 'ami-01e54781e37f571d9',
      'eu-west-2': 'ami-050d9b1c965f562e8',
      'eu-west-1': 'ami-00f2dfae76a21353a',
      'ap-northeast-3': 'ami-076bce7d0a31d31bc',
      'ap-northeast-2': 'ami-059c136482983f528',
      'ap-northeast-1': 'ami-027db2cfd97651e24',
      'ca-central-1': 'ami-00656ac213f63cdad',
      'sa-east-1': 'ami-018c49638e0b5bb13',
      'ap-southeast-1': 'ami-0d170699bd6cf63c8',
      'ap-southeast-2': 'ami-018c9a8f5d2f7c7db',
      'eu-central-1': 'ami-08550905cb42af2a4',
      'us-east-1': 'ami-06f0dec14b589d0c8',
      'us-east-2': 'ami-02db9e4dd67d28758',
      'us-west-1': 'ami-03c7d856f09bfb2b7',
      'us-west-2': 'ami-0ce85c8d26b5f523b',
    },
    cpuArchitecture: 'x64',
    baseImageType: 'ubuntu-20.04-cuda',
    supportedInstanceTypes: [
      'g5.xlarge',
      'g5.2xlarge',
      'g5.4xlarge',
      'g5.8xlarge',
      'g5.16xlarge',
      'g5.12xlarge',
      'g5.24xlarge',
      'g5.48xlarge',
      'p3.2xlarge',
      'p3.8xlarge',
      'p3.16xlarge',
    ],
    sshUsername: 'ubuntu',
    rootVolumeDeviceName: '/dev/sda1',
    rootVolumeSize: 125, // This is 50 GiB more than this AMI's default root volume size of 75 GiB.
  },
  {
    ids: {
      'ap-south-1': 'ami-03bead2079e3d3387',
      'eu-north-1': 'ami-0f890a3229d414597',
      'eu-west-3': 'ami-0a9de0d8ffdeb8177',
      'eu-west-2': 'ami-0b8ef550d8b9c2b05',
      'eu-west-1': 'ami-0c2c75c697f06fd25',
      'ap-northeast-3': 'ami-0ce4461841d4966a2',
      'ap-northeast-2': 'ami-0de1a8a0fb23b7973',
      'ap-northeast-1': 'ami-0f40e81ffb8012e05',
      'ca-central-1': 'ami-0d99e23e5e5e6d474',
      'sa-east-1': 'ami-094ccb45db76f8fb9',
      'ap-southeast-1': 'ami-085561f30b506ed45',
      'ap-southeast-2': 'ami-0e4f98ec100a04982',
      'eu-central-1': 'ami-07a947d70c05817c5',
      'us-east-1': 'ami-0d8667b0f72471655',
      'us-east-2': 'ami-0eff07ed600dfef97',
      'us-west-1': 'ami-08f85a7c70723e5d7',
      'us-west-2': 'ami-0e575809a4bbc7dd1',
    },
    cpuArchitecture: 'x64',
    baseImageType: 'ubuntu-20.04-cuda',
    supportedInstanceTypes: ['p4d.24xlarge', 'p5.48xlarge'],
    sshUsername: 'ubuntu',
    rootVolumeDeviceName: '/dev/sda1',
    rootVolumeSize: 125, // This is 50 GiB more than this AMI's default root volume size of 75 GiB.
  },
]

export const DEFAULT_BASE_IMAGE_TYPE = 'debian-12'
