import { EC2Client } from '@aws-sdk/client-ec2'
import assert from 'node:assert'
import test, { describe, mock } from 'node:test'
import { waitForImageToBeAvailable } from './findOrBuildAuxVmImage'

void describe('waitForAmiToBeAvailable', () => {
  void test('should return the image ID if the image is available', async () => {
    mock.method(EC2Client.prototype, 'send', async () => ({
      Images: [{ ImageId: 'ami-12345678', State: 'available' }],
    }))

    const imageId = await waitForImageToBeAvailable('my-image-name')
    assert.strictEqual(imageId, 'ami-12345678')
  })

  void test('should return null if the image does not exist', async () => {
    mock.method(EC2Client.prototype, 'send', async () => ({
      Images: [],
    }))

    const imageId = await waitForImageToBeAvailable('my-image-name')
    assert.strictEqual(imageId, null)
  })
})
