import subprocess
from templates import prompt_to_search
import sys
from IPython.core.interactiveshell import InteractiveShell
from io import StringIO
from IPython.utils.io import Tee
import re
import os
import random
import math
import time

ipython_shell = InteractiveShell.instance()
ipython_shell.colors = "nocolor"

# setup bash environment
os.system("bash -c \"echo '/home/agent' > ~/.last_dir; declare -p > ~/.last_env\"")


class CapturePrintDuring(object):
    def __init__(self, channel="stdout", suppress=True):
        self.channel = channel
        self.suppress = suppress
        self.printed = ""

    def __enter__(self):
        self.orig_stream = getattr(sys, self.channel)
        self.buffer = StringIO()
        self.tee = Tee(self.buffer, channel=self.channel)
        setattr(sys, self.channel, self.buffer if self.suppress else self.tee)

    def __exit__(self, etype, value, traceback):
        try:
            if value is not None:
                # If an error was raised, don't check anything else
                return False
            self.tee.flush()
            setattr(sys, self.channel, self.orig_stream)
            printed = self.buffer.getvalue()
            self.printed = printed
        finally:
            self.tee.close()


def sanitize_null_bytes(text):
    result = text.replace("\u0000", "")
    if result != text:
        print("WARNING: sanitized null bytes from text")
    return result


def maybe_prompt_to_search(text: str):
    if len(text) > 1500 * 3:
        filename = (
            f"long_output_{str(1 + math.floor(random.random() * 9007199254740990))}.txt"
        )
        open(filename, "w").write(text)
        return (
            prompt_to_search.replace("{{&filename}}", filename)
            .replace("{{&output_start}}", text[:100])
            .replace("{{&output_end}}", text[-100:])
        )
    return text


def _run_python(code):
    global ipython_shell
    pystarttime = time.time()
    cap = CapturePrintDuring()
    cap.__enter__()
    ipython_shell.run_cell(code)
    cap.__exit__(None, None, None)
    print("\nRunning python took", time.time() - pystarttime, "seconds", flush=True)
    ansi_escape = re.compile(r"\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])")
    printed = ansi_escape.sub("", cap.printed)
    print_result = re.sub(r"\n(Out|In)\[[0-9]+\]: ", "\n", printed)
    return maybe_prompt_to_search(
        "Python output: ```\n" + sanitize_null_bytes(print_result) + "\n```"
    )


def _run_bash(command):
    full_command = f" cd $( cat ~/.last_dir ) >/dev/null; source ~/.last_env 2> /dev/null && {command}; pwd > ~/.last_dir; declare -p > ~/.last_env"
    try:
        result = subprocess.run(
            ["bash", "-c", full_command],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            timeout=300,
        )
        output = sanitize_null_bytes(
            result.stdout.decode("utf-8", "replace")
            + result.stderr.decode("utf-8", "replace")
        )
    except subprocess.TimeoutExpired:
        output = "Command timed out."
    return maybe_prompt_to_search("Bash output: ```\n" + output + "\n```")
