import { execSync } from 'child_process'
import yargs from 'yargs'
import { hideBin } from 'yargs/helpers'
import { destroyAuxVm } from 'task-standard-drivers/src/aws'

async function main() {
  const argv = await yargs(hideBin(process.argv))
    .usage('Usage: $0 <taskEnvironmentIdentifier>')
    .command('$0 <taskEnvironmentIdentifier>', 'Destroy the given task environment', yargs => {
      yargs.positional('taskEnvironmentIdentifier', {
        describe: 'Task environment identifier',
        type: 'string',
      })
    })
    .help().argv

  const taskEnvironmentIdentifier = argv.taskEnvironmentIdentifier as string

  try {
    execSync(`docker rm -f ${taskEnvironmentIdentifier}`, { stdio: 'pipe' })
  } catch (e) {
    if (!e.message.includes('No such container')) {
      throw e
    }
  }

  try {
    execSync(`docker network rm ${taskEnvironmentIdentifier}-network`, { stdio: 'pipe' })
  } catch (e) {
    if (!e.message.includes(`network ${taskEnvironmentIdentifier}-network not found`)) {
      throw e
    }
  }

  // TODO: We should clean up any iptables rules that were created for the task environment.
  // This is tricky because npm run destroy doesn't have access to the --docker-host-ssh-destination argument
  // passed to npm run task.
  // A non-exhaustive list of possible solutions:
  //   1. Add a --docker-host-ssh-destination argument to npm run destroy
  //   2. Store the value of --docker-host-ssh-destination in a file in the task environment
  //   3. Store the value of --docker-host-ssh-destination as a tag on the task environment's Docker network

  await destroyAuxVm(taskEnvironmentIdentifier)
}

main().catch(console.error)
