import { DriverImpl } from 'task-standard-drivers/DriverImpl'

export async function getTaskSetupData(driver: DriverImpl, taskFamilyName: string, taskName: string) {
  const getTaskSetupDataResult = await driver.getTaskSetupData()
  switch (getTaskSetupDataResult.status) {
    case 'taskNotFound':
      throw new Error(`Task family ${taskFamilyName} does not contain a task named ${taskName}.`)
    case 'parseFailed':
      throw new Error(getTaskSetupDataResult.message)
    case 'processFailed':
      const { exitStatus, stdout, stderr } = getTaskSetupDataResult.execResult
      throw new Error(
        `Error getting task setup data.\n\nExit status: ${exitStatus}\n\nStdout: ${stdout}\n\nStderr: ${stderr}`,
      )
  }
  return getTaskSetupDataResult.taskSetupData
}
