import { execFileSync } from 'child_process'
import fs from 'fs'
import { mkdtemp, rm, writeFile } from 'fs/promises'
import os from 'os'
import path from 'path'
import { AuxVmDetails, Env } from 'task-standard-drivers/Driver'

export function header(text: string) {
  const green = '\x1b[32m'
  const reset = '\x1b[0m'
  console.log(green, `=== ${text} ===`, reset)
}

export type TaskEnvironmentData = {
  taskFamilyName: string
  taskName: string
  env: Env
  auxVMDetails: AuxVmDetails | null
}

export async function copyTaskEnvironmentDataToContainer(
  containerName: string,
  taskEnvironmentData: TaskEnvironmentData,
) {
  const tempDir = await mkdtemp(path.join(os.tmpdir(), containerName))
  const tempFile = path.join(tempDir, 'task-environment-data.json')
  await writeFile(tempFile, JSON.stringify(taskEnvironmentData))
  try {
    execFileSync('docker', ['cp', tempFile, `${containerName}:/root/.task-environment-data.json`])
  } finally {
    await rm(tempDir, { recursive: true })
  }
}

export function getTaskEnvironmentDataFromContainer(tempDir: string, containerName: string): TaskEnvironmentData {
  const tempFile = path.join(tempDir, 'task-environment-data.json')
  execFileSync('docker', ['cp', `${containerName}:/root/.task-environment-data.json`, tempFile])
  return JSON.parse(fs.readFileSync(tempFile).toString())
}
